@@ -230,6 +230,22 @@ class PipelineBuilder(ProtobufSerialisable):
 
         return task_request
 
+    def exceptionally(self, exception_task, *args, **kwargs) -> 'PipelineBuilder':
+        """
+        Adds finally to the pipeline
+
+        :param exception_task: the python function which should be decorated with @tasks.bind()
+        :param args: the task args
+        :param kwargs: the task kwargs
+        :return: the builder
+        """
+        try:
+            task = exception_task.to_task(args, kwargs, is_exceptionally=True)
+            self._pipeline.append(task)
+        except AttributeError:
+            raise AttributeError(f'Function {exception_task.__module__}.{exception_task.__name__} should be decorated with tasks.bind')
+        return self
+
     def finally_do(self, finally_action, *args, **kwargs) -> 'PipelineBuilder':
         """
         Adds finally to the pipeline
@@ -329,9 +345,13 @@ class PipelineBuilder(ProtobufSerialisable):
 
         finally_tasks = [task for task in all_tasks if task.is_finally]
         if len(finally_tasks) > 1:
-            errors.append('Cannot have more than one "finally_do" method')
+            errors.append('Cannot have more than one "finally_do" task')
         if len(finally_tasks) == 1 and finally_tasks[0] != self._pipeline[-1]:
-            errors.append('"finally_do" must be called at the end of a pipeline')
+            errors.append('"finally_do" must be called at the end of the pipeline')
+
+        exceptionally_tasks = [task for task in all_tasks if task.is_exceptionally]                
+        if any(list(set(exceptionally_tasks) & set(finally_tasks))):
+            errors.append('Tasks can be either finally_do or exceptionally but not both')
 
         if any(errors):
             error = ', '.join(errors)
@@ -339,6 +359,7 @@ class PipelineBuilder(ProtobufSerialisable):
 
         return self
 
+
     def to_proto(self, serialiser=None) -> Pipeline:
         """
         Serialises the pipeline to protobuf
