@@ -1,6 +1,11 @@
 from statefun_tasks.context import TaskContext
 from statefun_tasks.messages_pb2 import TaskResult, TupleOfAny
 from statefun_tasks.serialisation import pack_any
+from statefun_tasks.types import MessageSizeExceeded
+from statefun_tasks.type_helpers import _create_task_exception
+import logging
+
+_log = logging.getLogger('FlinkTasks')
 
 
 class ResultEmitter(object):
@@ -9,7 +14,9 @@ class ResultEmitter(object):
     def __init__(self):
         pass
 
-    def emit_result(self, context: TaskContext, task_request, task_result_or_exception):
+    async def emit_result(self, context: TaskContext, task_request, task_result_or_exception, pipeline: '_Pipeline'):
+        await pipeline.events.notify_emit_result(context, task_result_or_exception)
+
         # the result of this task is the result of the pipeline
         if isinstance(task_result_or_exception, TaskResult):
 
@@ -24,7 +31,7 @@ class ResultEmitter(object):
 
         # either send a message to egress if reply_topic was specified
         if task_request.HasField('reply_topic'):
-            context.pack_and_send_egress(topic=task_request.reply_topic, value=task_result_or_exception)
+            self._send_egress_message(context, task_request, task_result_or_exception)
 
         # or call back to a particular flink function if reply_address was specified
         elif task_request.HasField('reply_address'):
@@ -33,5 +40,11 @@ class ResultEmitter(object):
 
         # or call back to our caller (if there is one)
         elif context.pipeline_state.caller_id is not None:
-            if context.pipeline_state.caller_id != context.get_caller_id():  # don't call back to self
-                context.pack_and_send(context.pipeline_state.caller_address, context.pipeline_state.caller_id, task_result_or_exception)
+            context.pack_and_send(context.pipeline_state.caller_address, context.pipeline_state.caller_id, task_result_or_exception)
+
+    def _send_egress_message(self, context, task_request, task_result_or_exception):
+        try:
+            context.pack_and_send_egress(topic=task_request.reply_topic, value=task_result_or_exception)
+        except MessageSizeExceeded as e:
+            _log.warning(f'Unable to send egress message - {e}')
+            context.pack_and_send_egress(topic=task_request.reply_topic, value=_create_task_exception(task_request, e))
