@@ -30,7 +30,7 @@ class Task:
 
     @staticmethod
     def from_fields(task_id, task_type, task_args, task_kwargs, is_finally=None, namespace=None, worker_name=None,
-                    is_fruitful=None, retry_policy=None, display_name=None, is_wait=None, uid=None, **kwargs):
+                    is_fruitful=None, retry_policy=None, display_name=None, is_wait=None, uid=None, is_exceptionally=None, **kwargs):
         proto = TaskEntry(
             task_id=task_id,
             task_type=task_type,
@@ -42,7 +42,8 @@ class Task:
             retry_policy=retry_policy,
             display_name=display_name,
             is_wait=is_wait,
-            uid=uid if uid is not None else _gen_id()
+            uid=uid if uid is not None else _gen_id(),
+            is_exceptionally = is_exceptionally
         )
         return Task(proto, task_args, task_kwargs)
 
@@ -85,6 +86,13 @@ class Task:
         """
         return self._proto.complete
 
+    @property
+    def is_exceptionally(self):
+        """
+        Whether this task is an exceptionally task
+        """
+        return self._proto.is_exceptionally
+
     @property
     def is_finally(self):
         """
@@ -184,8 +192,8 @@ class Task:
 
         return self.task_id, self.task_type, self._args, self._kwargs
 
-    def mark_complete(self):
-        self._proto.complete = True
+    def mark_complete(self, value=True):
+        self._proto.complete = value
 
     def is_complete(self):
         return self._proto.complete
